from __future__ import annotations

import json
from typing import Any, List, Tuple, Literal, Mapping, Optional, cast, overload
from pathlib import Path

import httpx

from ..types import (
    TestCaseFiltersParam,
    TestCaseOrderByParam,
    DatasetSearchTestCasesParams,
)
from .._types import Body, Omit, Query, Headers, NotGiven, FileTypes, SequenceNotStr, omit, not_given
from .._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.common import APIResponse, TaskProgressParam, APIPaginatedMetadata, APIPaginatedResponse
from ..types.dataset import (
    Dataset,
    DatasetListParams,
    DatasetCreateParams,
    DatasetImportParams,
    DatasetUpdateParams,
    DatasetBulkDeleteParams,
    DatasetGenerateDocumentBasedParams,
    DatasetGenerateScenarioBasedParams,
)
from ..types.test_case import TestCase

__all__ = ["DatasetsResource", "AsyncDatasetsResource"]


class DatasetsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return DatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return DatasetsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        project_id: str,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Dataset:
        """Create a new empty dataset in the specified project.

        Parameters
        ----------
        name : str
            Name of the dataset to create.
        project_id : str
            Project ID to create the dataset in.
        description : Optional[str]
            Description of the dataset to create.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Dataset
            The newly created dataset.
        """
        response = self._post(
            "/v2/datasets",
            body=maybe_transform(
                {
                    "name": name,
                    "project_id": project_id,
                    "description": description,
                },
                DatasetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Dataset],
        )

        return self._unwrap(response)

    def upload(
        self,
        *,
        project_id: str,
        data: FileTypes | list[dict[str, Any]] | str,
        dataset_id: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Dataset:
        """Upload a dataset from a file or a list of dictionaries.

        Parameters
        ----------
        project_id : str
            Project ID to import the dataset into.
        data : FileTypes | list[dict[str, Any]] | str
            Data to upload.
        dataset_id : Optional[str]
            Dataset ID to update (optional).
        name : Optional[str]
            Name of the dataset (optional).

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Dataset
            The uploaded dataset.
        """
        if isinstance(data, str):
            data = Path(data)
        if isinstance(data, list):
            data = ("test_cases.json", json.dumps(data).encode("utf-8"))

        body = deepcopy_minimal(
            {
                "file": data,
            }
        )

        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])

        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}

        response = self._post(
            "/v2/datasets/import",
            body=maybe_transform(body, DatasetImportParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "project_id": project_id,
                        "dataset_id": dataset_id,
                        "name": name,
                    },
                    DatasetImportParams,
                ),
            ),
            cast_to=APIResponse[Dataset],
        )

        return self._unwrap(response)

    def retrieve(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Dataset:
        """Retrieve a dataset by its ID.

        Parameters
        ----------
        dataset_id : str
            ID of the dataset to retrieve.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Dataset
            The retrieved dataset.

        Raises
        ------
        ValueError
            If ``dataset_id`` is empty.
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        response = self._get(
            f"/v2/datasets/{dataset_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Dataset],
        )

        return self._unwrap(response)

    def update(
        self,
        dataset_id: str,
        *,
        description: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        status: Optional[TaskProgressParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Dataset:
        """Update an existing dataset's metadata.

        Parameters
        ----------
        dataset_id : str
            ID of the dataset to update.
        description : Optional[str]
            Description of the dataset to update.
        name : Optional[str]
            Name of the dataset to update.
        status : Optional[TaskProgressParam]
            Status of the dataset to update.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Dataset
            The updated dataset.

        Raises
        ------
        ValueError
            If ``dataset_id`` is empty.
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        response = self._patch(
            f"/v2/datasets/{dataset_id}",
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "status": status,
                },
                DatasetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Dataset],
        )

        return self._unwrap(response)

    def list(
        self,
        *,
        project_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[Dataset]:
        """List all datasets, optionally filtered by project.

        Parameters
        ----------
        project_id : Optional[str]
            Project ID to list datasets for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[Dataset]
            A list of datasets.
        """
        response = self._get(
            "/v2/datasets",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"project_id": project_id}, DatasetListParams),
            ),
            cast_to=APIResponse[List[Dataset]],
        )

        return self._unwrap(response)

    def delete(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a dataset by its ID.

        Parameters
        ----------
        dataset_id : str
            ID of the dataset to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``dataset_id`` is empty.
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        response = self._delete(
            f"/v2/datasets/{dataset_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def bulk_delete(
        self,
        *,
        dataset_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete multiple datasets at once.

        Parameters
        ----------
        dataset_ids : SequenceNotStr[str]
            IDs of the datasets to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None
        """
        response = self._delete(
            "/v2/datasets",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"dataset_ids": dataset_ids}, DatasetBulkDeleteParams),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def generate_scenario_based(
        self,
        *,
        project_id: str,
        agent_id: str,
        scenario_id: str,
        n_examples: int | Omit = omit,
        dataset_id: Optional[str] | Omit = omit,
        dataset_name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Dataset:
        """Generate a dataset of test cases from scenario definitions.

        Parameters
        ----------
        project_id : str
            The ID of the project.
        agent_id : str
            The ID of the agent to use for generation.
        scenario_id : str
            The ID of the scenario to use.
        n_examples : int
            Total number of examples to generate.
        dataset_id : Optional[str]
            The ID of the dataset to use (required when dataset_name is not provided).
        dataset_name : Optional[str]
            Name for the generated dataset.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Dataset
            The generated dataset.

        Raises
        ------
        ValueError
            If neither ``dataset_id`` nor ``dataset_name`` is provided.
        """

        if dataset_id is omit and dataset_name is omit:
            raise ValueError("'dataset_name' is required when 'dataset_id' is not provided")

        response = self._post(
            "/v2/datasets/generate-scenario-based",
            body=maybe_transform(
                {
                    "agent_id": agent_id,
                    "project_id": project_id,
                    "scenario_id": scenario_id,
                    "dataset_name": dataset_name,
                    "num_examples": n_examples,
                    "dataset_id": dataset_id,
                },
                DatasetGenerateScenarioBasedParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Dataset],
        )

        return self._unwrap(response)

    def generate_document_based(
        self,
        *,
        agent_id: str,
        knowledge_base_id: str,
        project_id: str,
        dataset_name: str | Omit = omit,
        description: Optional[str] | Omit = omit,
        n_examples: int | Omit = omit,
        topic_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Dataset:
        """Generate a dataset of test cases from knowledge base documents.

        Parameters
        ----------
        agent_id : str
            The ID of the agent to use for generation.
        knowledge_base_id : str
            The ID of the knowledge base to use for generation.
        project_id : str
            The ID of the project to use for generation.
        dataset_name : str
            Name for the generated dataset.
        description : Optional[str]
            Description of the generated dataset.
        n_examples : int
            Total number of examples to generate.
        topic_ids : SequenceNotStr[str]
            IDs of the topics to use for generation.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Dataset
            The generated dataset.
        """
        response = self._post(
            "/v2/datasets/generate-document-based",
            body=maybe_transform(
                {
                    "agent_id": agent_id,
                    "knowledge_base_id": knowledge_base_id,
                    "project_id": project_id,
                    "dataset_name": dataset_name,
                    "description": description,
                    "n_examples": n_examples,
                    "topic_ids": topic_ids,
                },
                DatasetGenerateDocumentBasedParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Dataset],
        )

        return self._unwrap(response)

    def list_tags(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[str]:
        """List all tags associated with a dataset.

        Parameters
        ----------
        dataset_id : str
            The ID of the dataset to list tags for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[str]
            A list of tag strings.

        Raises
        ------
        ValueError
            If ``dataset_id`` is empty.
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        response = self._get(
            f"/v2/datasets/{dataset_id}/tags",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[List[str]],
        )

        return self._unwrap(response)

    def list_test_cases(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[TestCase]:
        """List all test cases belonging to a dataset.

        Fetches every page via :meth:`search_test_cases` (same as an unfiltered search).

        Parameters
        ----------
        dataset_id : str
            The ID of the dataset to list test cases for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[TestCase]
            A list of test cases.

        Raises
        ------
        ValueError
            If ``dataset_id`` is empty.
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        page_limit = 100
        all_items: List[TestCase] = []
        offset = 0
        while True:
            page, meta = self.search_test_cases(
                dataset_id,
                limit=page_limit,
                offset=offset,
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                include_metadata=True,
            )
            all_items.extend(page)
            next_offset = meta.offset + meta.count
            if next_offset >= meta.total or not page:
                break
            offset = next_offset
        return all_items

    @overload
    def search_test_cases(
        self,
        dataset_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[TestCaseOrderByParam]] | Omit = omit,
        filters: Optional[TestCaseFiltersParam] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: Literal[False] = False,
    ) -> List[TestCase]: ...

    @overload
    def search_test_cases(
        self,
        dataset_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[TestCaseOrderByParam]] | Omit = omit,
        filters: Optional[TestCaseFiltersParam] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: Literal[True] = True,
    ) -> Tuple[List[TestCase], APIPaginatedMetadata]: ...

    def search_test_cases(
        self,
        dataset_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[TestCaseOrderByParam]] | Omit = omit,
        filters: Optional[TestCaseFiltersParam] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[TestCase] | Tuple[List[TestCase], APIPaginatedMetadata]:
        """Search test cases in a dataset using filters, sorting, and pagination.

        Parameters
        ----------
        dataset_id : str
            The ID of the dataset to search test cases in.
        query : Optional[str]
            Search query for test cases.
        order_by : Optional[List[TestCaseOrderByParam]]
            Order by criteria for test cases.
        filters : Optional[TestCaseFiltersParam]
            Search filters to apply.
        limit : int
            Maximum number of results to return.
        offset : int
            Number of results to skip.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[TestCase] | Tuple[List[TestCase], APIPaginatedMetadata]
            A list of matching test cases, optionally with pagination metadata.

        Raises
        ------
        ValueError
            If ``dataset_id`` is empty.
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        response = self._post(
            f"/v2/datasets/{dataset_id}/test-cases/search",
            body=maybe_transform(
                {
                    "filters": filters,
                    "order_by": order_by,
                    "search": query,
                },
                DatasetSearchTestCasesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"limit": limit, "offset": offset},
                    DatasetSearchTestCasesParams,
                ),
            ),
            cast_to=APIPaginatedResponse[TestCase, None],
        )

        return self._unwrap_paginated(response, include_metadata)


class AsyncDatasetsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AsyncDatasetsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        project_id: str,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Dataset:
        """Create a new empty dataset in the specified project.

        Parameters
        ----------
        name : str
            Name of the dataset to create.
        project_id : str
            Project ID to create the dataset in.
        description : Optional[str]
            Description of the dataset to create.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Dataset
            The newly created dataset.
        """
        response = await self._post(
            "/v2/datasets",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "project_id": project_id,
                    "description": description,
                },
                DatasetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Dataset],
        )

        return self._unwrap(response)

    async def upload(
        self,
        *,
        project_id: str,
        data: FileTypes | list[dict[str, Any]] | str,
        dataset_id: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Dataset:
        """Upload a dataset from a file or a list of dictionaries.

        Parameters
        ----------
        project_id : str
            Project ID to import the dataset into.
        data : FileTypes | list[dict[str, Any]] | str
            Data to upload.
        dataset_id : Optional[str]
            Dataset ID to update (optional).
        name : Optional[str]
            Name of the dataset (optional).

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Dataset
            The uploaded dataset.
        """
        if isinstance(data, str):
            data = Path(data)
        if isinstance(data, list):
            data = ("test_cases.json", json.dumps(data).encode("utf-8"))

        body = deepcopy_minimal(
            {
                "file": data,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        response = await self._post(
            "/v2/datasets/import",
            body=await async_maybe_transform(body, DatasetImportParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "project_id": project_id,
                        "dataset_id": dataset_id,
                        "name": name,
                    },
                    DatasetImportParams,
                ),
            ),
            cast_to=APIResponse[Dataset],
        )

        return self._unwrap(response)

    async def retrieve(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Dataset:
        """Retrieve a dataset by its ID.

        Parameters
        ----------
        dataset_id : str
            ID of the dataset to retrieve.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Dataset
            The retrieved dataset.

        Raises
        ------
        ValueError
            If ``dataset_id`` is empty.
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        response = await self._get(
            f"/v2/datasets/{dataset_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Dataset],
        )

        return self._unwrap(response)

    async def update(
        self,
        dataset_id: str,
        *,
        description: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        status: Optional[TaskProgressParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Dataset:
        """Update an existing dataset's metadata.

        Parameters
        ----------
        dataset_id : str
            ID of the dataset to update.
        description : Optional[str]
            Description of the dataset to update.
        name : Optional[str]
            Name of the dataset to update.
        status : Optional[TaskProgressParam]
            Status of the dataset to update.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Dataset
            The updated dataset.

        Raises
        ------
        ValueError
            If ``dataset_id`` is empty.
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        response = await self._patch(
            f"/v2/datasets/{dataset_id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "status": status,
                },
                DatasetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Dataset],
        )

        return self._unwrap(response)

    async def list(
        self,
        *,
        project_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[Dataset]:
        """List all datasets, optionally filtered by project.

        Parameters
        ----------
        project_id : Optional[str]
            Project ID to list datasets for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[Dataset]
            A list of datasets.
        """
        response = await self._get(
            "/v2/datasets",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"project_id": project_id}, DatasetListParams),
            ),
            cast_to=APIResponse[List[Dataset]],
        )

        return self._unwrap(response)

    async def delete(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a dataset by its ID.

        Parameters
        ----------
        dataset_id : str
            ID of the dataset to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``dataset_id`` is empty.
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        response = await self._delete(
            f"/v2/datasets/{dataset_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def bulk_delete(
        self,
        *,
        dataset_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete multiple datasets at once.

        Parameters
        ----------
        dataset_ids : SequenceNotStr[str]
            IDs of the datasets to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None
        """
        response = await self._delete(
            "/v2/datasets",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"dataset_ids": dataset_ids}, DatasetBulkDeleteParams),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def generate_scenario_based(
        self,
        *,
        project_id: str,
        agent_id: str,
        scenario_id: str,
        n_examples: int | Omit = omit,
        dataset_id: Optional[str] | Omit = omit,
        dataset_name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Dataset:
        """Generate a dataset of test cases from scenario definitions.

        Parameters
        ----------
        project_id : str
            The ID of the project.
        agent_id : str
            The ID of the agent to use for generation.
        scenario_id : str
            The ID of the scenario to use.
        n_examples : int
            Total number of examples to generate.
        dataset_id : Optional[str]
            The ID of the dataset to use (required when dataset_name is not provided).
        dataset_name : Optional[str]
            Name for the generated dataset.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Dataset
            The generated dataset.

        Raises
        ------
        ValueError
            If neither ``dataset_id`` nor ``dataset_name`` is provided.
        """

        if dataset_id is omit and dataset_name is omit:
            raise ValueError("'dataset_name' is required when 'dataset_id' is not provided")

        response = await self._post(
            "/v2/datasets/generate-scenario-based",
            body=await async_maybe_transform(
                {
                    "agent_id": agent_id,
                    "project_id": project_id,
                    "scenario_id": scenario_id,
                    "dataset_name": dataset_name,
                    "num_examples": n_examples,
                    "dataset_id": dataset_id,
                },
                DatasetGenerateScenarioBasedParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Dataset],
        )

        return self._unwrap(response)

    async def generate_document_based(
        self,
        *,
        agent_id: str,
        knowledge_base_id: str,
        project_id: str,
        dataset_name: str | Omit = omit,
        description: Optional[str] | Omit = omit,
        n_examples: int | Omit = omit,
        topic_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Dataset:
        """Generate a dataset of test cases from knowledge base documents.

        Parameters
        ----------
        agent_id : str
            The ID of the agent to use for generation.
        knowledge_base_id : str
            The ID of the knowledge base to use for generation.
        project_id : str
            The ID of the project to use for generation.
        dataset_name : str
            Name for the generated dataset.
        description : Optional[str]
            Description of the generated dataset.
        n_examples : int
            Total number of examples to generate.
        topic_ids : SequenceNotStr[str]
            IDs of the topics to use for generation.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Dataset
            The generated dataset.
        """
        response = await self._post(
            "/v2/datasets/generate-document-based",
            body=await async_maybe_transform(
                {
                    "agent_id": agent_id,
                    "knowledge_base_id": knowledge_base_id,
                    "project_id": project_id,
                    "dataset_name": dataset_name,
                    "description": description,
                    "n_examples": n_examples,
                    "topic_ids": topic_ids,
                },
                DatasetGenerateDocumentBasedParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Dataset],
        )

        return self._unwrap(response)

    async def list_tags(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[str]:
        """List all tags associated with a dataset.

        Parameters
        ----------
        dataset_id : str
            The ID of the dataset to list tags for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[str]
            A list of tag strings.

        Raises
        ------
        ValueError
            If ``dataset_id`` is empty.
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        response = await self._get(
            f"/v2/datasets/{dataset_id}/tags",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[List[str]],
        )

        return self._unwrap(response)

    async def list_test_cases(
        self,
        dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[TestCase]:
        """List all test cases belonging to a dataset.

        Fetches every page via :meth:`search_test_cases` (same as an unfiltered search).

        Parameters
        ----------
        dataset_id : str
            The ID of the dataset to list test cases for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[TestCase]
            A list of test cases.

        Raises
        ------
        ValueError
            If ``dataset_id`` is empty.
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        page_limit = 100
        all_items: List[TestCase] = []
        offset = 0
        while True:
            page, meta = await self.search_test_cases(
                dataset_id,
                limit=page_limit,
                offset=offset,
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                include_metadata=True,
            )
            all_items.extend(page)
            next_offset = meta.offset + meta.count
            if next_offset >= meta.total or not page:
                break
            offset = next_offset
        return all_items

    @overload
    async def search_test_cases(
        self,
        dataset_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[TestCaseOrderByParam]] | Omit = omit,
        filters: Optional[TestCaseFiltersParam] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: Literal[False] = False,
    ) -> List[TestCase]: ...

    @overload
    async def search_test_cases(
        self,
        dataset_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[TestCaseOrderByParam]] | Omit = omit,
        filters: Optional[TestCaseFiltersParam] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: Literal[True] = True,
    ) -> Tuple[List[TestCase], APIPaginatedMetadata]: ...

    async def search_test_cases(
        self,
        dataset_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[TestCaseOrderByParam]] | Omit = omit,
        filters: Optional[TestCaseFiltersParam] | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[TestCase] | Tuple[List[TestCase], APIPaginatedMetadata]:
        """Search test cases in a dataset using filters, sorting, and pagination.

        Parameters
        ----------
        dataset_id : str
            The ID of the dataset to search test cases in.
        query : Optional[str]
            Search query for test cases.
        order_by : Optional[List[TestCaseOrderByParam]]
            Order by criteria for test cases.
        filters : Optional[TestCaseFiltersParam]
            Search filters to apply.
        limit : int
            Maximum number of results to return.
        offset : int
            Number of results to skip.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[TestCase] | Tuple[List[TestCase], APIPaginatedMetadata]
            A list of matching test cases, optionally with pagination metadata.

        Raises
        ------
        ValueError
            If ``dataset_id`` is empty.
        """
        if not dataset_id:
            raise ValueError(f"Expected a non-empty value for `dataset_id` but received {dataset_id!r}")
        response = await self._post(
            f"/v2/datasets/{dataset_id}/test-cases/search",
            body=await async_maybe_transform(
                {
                    "filters": filters,
                    "order_by": order_by,
                    "search": query,
                },
                DatasetSearchTestCasesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"limit": limit, "offset": offset},
                    DatasetSearchTestCasesParams,
                ),
            ),
            cast_to=APIPaginatedResponse[TestCase, None],
        )

        return self._unwrap_paginated(response, include_metadata)


class DatasetsResourceWithRawResponse:
    def __init__(self, datasets: DatasetsResource) -> None:
        self._datasets = datasets

        self.create = to_raw_response_wrapper(
            datasets.create,
        )
        self.upload = to_raw_response_wrapper(
            datasets.upload,
        )
        self.retrieve = to_raw_response_wrapper(
            datasets.retrieve,
        )
        self.update = to_raw_response_wrapper(
            datasets.update,
        )
        self.list = to_raw_response_wrapper(
            datasets.list,
        )
        self.delete = to_raw_response_wrapper(
            datasets.delete,
        )
        self.bulk_delete = to_raw_response_wrapper(
            datasets.bulk_delete,
        )
        self.generate_scenario_based = to_raw_response_wrapper(
            datasets.generate_scenario_based,
        )
        self.generate_document_based = to_raw_response_wrapper(
            datasets.generate_document_based,
        )
        self.list_tags = to_raw_response_wrapper(
            datasets.list_tags,
        )
        self.list_test_cases = to_raw_response_wrapper(
            datasets.list_test_cases,
        )
        self.search_test_cases = to_raw_response_wrapper(
            datasets.search_test_cases,
        )


class AsyncDatasetsResourceWithRawResponse:
    def __init__(self, datasets: AsyncDatasetsResource) -> None:
        self._datasets = datasets

        self.create = async_to_raw_response_wrapper(
            datasets.create,
        )
        self.upload = async_to_raw_response_wrapper(
            datasets.upload,
        )
        self.retrieve = async_to_raw_response_wrapper(
            datasets.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            datasets.update,
        )
        self.list = async_to_raw_response_wrapper(
            datasets.list,
        )
        self.delete = async_to_raw_response_wrapper(
            datasets.delete,
        )
        self.bulk_delete = async_to_raw_response_wrapper(
            datasets.bulk_delete,
        )
        self.generate_scenario_based = async_to_raw_response_wrapper(
            datasets.generate_scenario_based,
        )
        self.generate_document_based = async_to_raw_response_wrapper(
            datasets.generate_document_based,
        )
        self.list_tags = async_to_raw_response_wrapper(
            datasets.list_tags,
        )
        self.list_test_cases = async_to_raw_response_wrapper(
            datasets.list_test_cases,
        )
        self.search_test_cases = async_to_raw_response_wrapper(
            datasets.search_test_cases,
        )


class DatasetsResourceWithStreamingResponse:
    def __init__(self, datasets: DatasetsResource) -> None:
        self._datasets = datasets

        self.create = to_streamed_response_wrapper(
            datasets.create,
        )
        self.upload = to_streamed_response_wrapper(
            datasets.upload,
        )
        self.retrieve = to_streamed_response_wrapper(
            datasets.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            datasets.update,
        )
        self.list = to_streamed_response_wrapper(
            datasets.list,
        )
        self.delete = to_streamed_response_wrapper(
            datasets.delete,
        )
        self.bulk_delete = to_streamed_response_wrapper(
            datasets.bulk_delete,
        )
        self.generate_scenario_based = to_streamed_response_wrapper(
            datasets.generate_scenario_based,
        )
        self.generate_document_based = to_streamed_response_wrapper(
            datasets.generate_document_based,
        )
        self.list_tags = to_streamed_response_wrapper(
            datasets.list_tags,
        )
        self.list_test_cases = to_streamed_response_wrapper(
            datasets.list_test_cases,
        )
        self.search_test_cases = to_streamed_response_wrapper(
            datasets.search_test_cases,
        )


class AsyncDatasetsResourceWithStreamingResponse:
    def __init__(self, datasets: AsyncDatasetsResource) -> None:
        self._datasets = datasets

        self.create = async_to_streamed_response_wrapper(
            datasets.create,
        )
        self.upload = async_to_streamed_response_wrapper(
            datasets.upload,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            datasets.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            datasets.update,
        )
        self.list = async_to_streamed_response_wrapper(
            datasets.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            datasets.delete,
        )
        self.bulk_delete = async_to_streamed_response_wrapper(
            datasets.bulk_delete,
        )
        self.generate_scenario_based = async_to_streamed_response_wrapper(
            datasets.generate_scenario_based,
        )
        self.generate_document_based = async_to_streamed_response_wrapper(
            datasets.generate_document_based,
        )
        self.list_tags = async_to_streamed_response_wrapper(
            datasets.list_tags,
        )
        self.list_test_cases = async_to_streamed_response_wrapper(
            datasets.list_test_cases,
        )
        self.search_test_cases = async_to_streamed_response_wrapper(
            datasets.search_test_cases,
        )



"""Dataset domain types."""

from typing import Any, Dict, List, Literal, Iterable, Optional, TypedDict
from datetime import datetime
from typing_extensions import Required

from .common import TaskState, OrderByParam, TaskProgress, FilterValueParam, TaskProgressParam
from .._types import FileTypes, SequenceNotStr
from .._models import BaseModel

__all__ = [
    "Dataset",
    "DatasetReference",
    "DatasetSubset",
    "DatasetSubsetParam",
    "DatasetListParams",
    "DatasetCreateParams",
    "DatasetImportParams",
    "DatasetUpdateParams",
    "DatasetBulkDeleteParams",
    "DatasetSearchTestCasesParams",
    "TestCaseOrderByParam",
    "TestCaseFiltersParam",
    "DatasetGenerateAdversarialParams",
    "DatasetGenerateDocumentBasedParams",
    "DatasetGenerateScenarioBasedParams",
]


# ---------------------------------------------------------------------------
# Models
# ---------------------------------------------------------------------------


class Dataset(BaseModel):
    id: str
    created_at: datetime
    description: Optional[str] = None
    name: str
    project_id: str
    status: TaskProgress
    tags: List[str]
    updated_at: datetime

    @property
    def state(self) -> TaskState:
        return self.status.state


class DatasetReference(BaseModel):
    id: str
    name: str


class DatasetSubset(BaseModel):
    dataset_id: str
    tags: Optional[List[str]] = None
    target_type: Optional[Literal["dataset"]] = None


class DatasetSubsetParam(TypedDict, total=False):
    dataset_id: Required[str]
    tags: Optional[SequenceNotStr[str]]
    target_type: Literal["dataset"]


# ---------------------------------------------------------------------------
# Params
# ---------------------------------------------------------------------------


class DatasetListParams(TypedDict, total=False):
    project_id: Optional[str]


class DatasetCreateParams(TypedDict, total=False):
    name: Required[str]
    project_id: Required[str]
    description: Optional[str]


class DatasetImportParams(TypedDict, total=False):
    file: Required[FileTypes]
    project_id: Required[str]
    dataset_id: Optional[str]
    name: Optional[str]


class DatasetUpdateParams(TypedDict, total=False):
    description: Optional[str]
    name: Optional[str]
    status: Optional[TaskProgressParam]


class DatasetBulkDeleteParams(TypedDict, total=False):
    dataset_ids: Required[SequenceNotStr[str]]


# ---------------------------------------------------------------------------
# Search params
# ---------------------------------------------------------------------------

TestCaseSortColumn = Literal["created_at", "id", "status", "updated_at"]
TestCaseFilterColumn = Literal["metrics", "status", "tags"]

TestCaseOrderByParam = OrderByParam[TestCaseSortColumn]
TestCaseFiltersParam = Dict[TestCaseFilterColumn, FilterValueParam]


class DatasetSearchTestCasesParams(TypedDict, total=False):
    search: Optional[str]
    order_by: Optional[List[TestCaseOrderByParam]]
    filters: Optional[TestCaseFiltersParam]
    limit: int
    offset: int


# ---------------------------------------------------------------------------
# Generation params
# ---------------------------------------------------------------------------


class Category(TypedDict, total=False):
    desc: Required[str]
    name: Required[str]
    id: Optional[str]


class DatasetGenerateAdversarialParams(TypedDict, total=False):
    agent_id: Required[str]
    project_id: Required[str]
    categories: Iterable[Category]
    dataset_name: str
    description: Optional[str]
    n_examples_per_category: int


class DatasetGenerateDocumentBasedParams(TypedDict, total=False):
    agent_id: Required[str]
    knowledge_base_id: Required[str]
    project_id: Required[str]
    dataset_name: str
    description: Optional[str]
    n_examples: int
    topic_ids: SequenceNotStr[str]


class DatasetGenerateScenarioBasedParams(TypedDict, total=False):
    agent_id: Required[str]
    project_id: Required[str]
    scenario_id: Required[str]
    dataset_name: Optional[str]
    n_examples: int
    scenario_config: Optional[Dict[str, Any]]



import importlib.metadata

import giskard_hub


def test_version_is_resolved():
    """Verify that __version__ is resolved from package metadata and is not 'unknown'."""
    pyproject_version = importlib.metadata.version("giskard-hub")
    assert giskard_hub.__version__ == pyproject_version
    assert giskard_hub.__version__ != "unknown"



from __future__ import annotations

import time
from typing import TYPE_CHECKING, Any, cast

import anyio

from ._response import BaseAPIResponse

if TYPE_CHECKING:
    from ._client import HubClient, AsyncHubClient


class SyncAPIResource:
    _client: HubClient

    def __init__(self, client: HubClient) -> None:
        self._client = client
        self._get = client.get
        self._post = client.post
        self._patch = client.patch
        self._put = client.put
        self._delete = client.delete
        self._get_api_list = client.get_api_list

    def _sleep(self, seconds: float) -> None:
        time.sleep(seconds)

    def _unwrap(self, result: Any) -> Any:
        """Unwrap a JSON:API response wrapper, returning ``result.data``.

        When the request was made via ``.with_raw_response`` or
        ``.with_streaming_response``, *result* is an HTTP-level
        ``BaseAPIResponse`` and is returned as-is so the raw-response
        contract is preserved.
        """
        if isinstance(result, BaseAPIResponse):
            return cast(Any, result)
        return result.data

    def _unwrap_paginated(self, result: Any, include_metadata: bool) -> Any:
        """Unwrap a paginated JSON:API response wrapper.

        When *include_metadata* is true the return value is a
        ``(data, metadata)`` tuple; otherwise only ``data`` is returned.

        Raw / streaming responses are passed through unchanged.
        """
        if isinstance(result, BaseAPIResponse):
            return cast(Any, result)
        if include_metadata:
            return result.data, result.metadata
        return result.data


class AsyncAPIResource:
    _client: AsyncHubClient

    def __init__(self, client: AsyncHubClient) -> None:
        self._client = client
        self._get = client.get
        self._post = client.post
        self._patch = client.patch
        self._put = client.put
        self._delete = client.delete
        self._get_api_list = client.get_api_list

    async def _sleep(self, seconds: float) -> None:
        await anyio.sleep(seconds)

    def _unwrap(self, result: Any) -> Any:
        """Unwrap a JSON:API response wrapper, returning ``result.data``.

        When the request was made via ``.with_raw_response`` or
        ``.with_streaming_response``, *result* is an HTTP-level
        ``BaseAPIResponse`` and is returned as-is so the raw-response
        contract is preserved.
        """
        if isinstance(result, BaseAPIResponse):
            return cast(Any, result)
        return result.data

    def _unwrap_paginated(self, result: Any, include_metadata: bool) -> Any:
        """Unwrap a paginated JSON:API response wrapper.

        When *include_metadata* is true the return value is a
        ``(data, metadata)`` tuple; otherwise only ``data`` is returned.

        Raw / streaming responses are passed through unchanged.
        """
        if isinstance(result, BaseAPIResponse):
            return cast(Any, result)
        if include_metadata:
            return result.data, result.metadata
        return result.data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, List, cast

import pytest

from giskard_hub import HubClient, AsyncHubClient
from tests.utils import assert_matches_type
from giskard_hub.types import Agent, AgentOutput, AgentDetectStatefulness

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAgents:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: HubClient) -> None:
        agent = client.agents.create(
            headers={"name": "value"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            supported_languages=["string"],
            url="url",
        )
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: HubClient) -> None:
        agent = client.agents.create(
            headers={"name": "value"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            supported_languages=["string"],
            url="url",
            description="description",
            stateful=True,
        )
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: HubClient) -> None:
        response = client.agents.with_raw_response.create(
            headers={"name": "value"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            supported_languages=["string"],
            url="url",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = response.parse()
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: HubClient) -> None:
        with client.agents.with_streaming_response.create(
            headers={"name": "value"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            supported_languages=["string"],
            url="url",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = response.parse()
            assert_matches_type(Agent, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: HubClient) -> None:
        agent = client.agents.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: HubClient) -> None:
        response = client.agents.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = response.parse()
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: HubClient) -> None:
        with client.agents.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = response.parse()
            assert_matches_type(Agent, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `agent_id` but received ''",
        ):
            client.agents.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: HubClient) -> None:
        agent = client.agents.update(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: HubClient) -> None:
        agent = client.agents.update(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            headers={"name": "value"},
            name="name",
            supported_languages=["string"],
            url="url",
        )
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: HubClient) -> None:
        response = client.agents.with_raw_response.update(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = response.parse()
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: HubClient) -> None:
        with client.agents.with_streaming_response.update(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = response.parse()
            assert_matches_type(Agent, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `agent_id` but received ''",
        ):
            client.agents.with_raw_response.update(
                agent_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: HubClient) -> None:
        agent = client.agents.list()
        assert_matches_type(List[Agent], agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: HubClient) -> None:
        agent = client.agents.list(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[Agent], agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: HubClient) -> None:
        response = client.agents.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = response.parse()
        assert_matches_type(List[Agent], agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: HubClient) -> None:
        with client.agents.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = response.parse()
            assert_matches_type(List[Agent], agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: HubClient) -> None:
        agent = client.agents.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(type(None), agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: HubClient) -> None:
        response = client.agents.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = response.parse()
        assert_matches_type(type(None), agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: HubClient) -> None:
        with client.agents.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = response.parse()
            assert_matches_type(type(None), agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `agent_id` but received ''",
        ):
            client.agents.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_bulk_delete(self, client: HubClient) -> None:
        agent = client.agents.bulk_delete(
            agent_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(type(None), agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_bulk_delete(self, client: HubClient) -> None:
        response = client.agents.with_raw_response.bulk_delete(
            agent_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = response.parse()
        assert_matches_type(type(None), agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_bulk_delete(self, client: HubClient) -> None:
        with client.agents.with_streaming_response.bulk_delete(
            agent_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = response.parse()
            assert_matches_type(type(None), agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_generate_completion(self, client: HubClient) -> None:
        agent = client.agents.generate_completion(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
        )
        assert_matches_type(AgentOutput, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_generate_completion(self, client: HubClient) -> None:
        response = client.agents.with_raw_response.generate_completion(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = response.parse()
        assert_matches_type(AgentOutput, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_generate_completion(self, client: HubClient) -> None:
        with client.agents.with_streaming_response.generate_completion(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = response.parse()
            assert_matches_type(AgentOutput, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_generate_completion(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `agent_id` but received ''",
        ):
            client.agents.with_raw_response.generate_completion(
                agent_id="",
                messages=[
                    {
                        "content": "content",
                        "role": "role",
                    }
                ],
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_test_connection(self, client: HubClient) -> None:
        agent = client.agents.test_connection(
            url="url",
        )
        assert_matches_type(AgentOutput, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_test_connection_with_all_params(self, client: HubClient) -> None:
        agent = client.agents.test_connection(
            url="url",
            headers={"foo": "string"},
        )
        assert_matches_type(AgentOutput, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_test_connection(self, client: HubClient) -> None:
        response = client.agents.with_raw_response.test_connection(
            url="url",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = response.parse()
        assert_matches_type(AgentOutput, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_test_connection(self, client: HubClient) -> None:
        with client.agents.with_streaming_response.test_connection(
            url="url",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = response.parse()
            assert_matches_type(AgentOutput, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_detect_statefulness(self, client: HubClient) -> None:
        result = client.agents.detect_statefulness(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(AgentDetectStatefulness, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_detect_statefulness(self, client: HubClient) -> None:
        response = client.agents.with_raw_response.detect_statefulness(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = response.parse()
        assert_matches_type(AgentDetectStatefulness, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_detect_statefulness(self, client: HubClient) -> None:
        with client.agents.with_streaming_response.detect_statefulness(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = response.parse()
            assert_matches_type(AgentDetectStatefulness, result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_detect_statefulness(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `agent_id` but received ''",
        ):
            client.agents.with_raw_response.detect_statefulness(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_generate_description(self, client: HubClient) -> None:
        agent = client.agents.generate_description(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(str, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_generate_description(self, client: HubClient) -> None:
        response = client.agents.with_raw_response.generate_description(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = response.parse()
        assert_matches_type(str, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_generate_description(self, client: HubClient) -> None:
        with client.agents.with_streaming_response.generate_description(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = response.parse()
            assert_matches_type(str, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_generate_description(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `agent_id` but received ''",
        ):
            client.agents.with_raw_response.generate_description(
                "",
            )


class TestAsyncAgents:
    parametrize = pytest.mark.parametrize(
        "async_client",
        [False, True, {"http_client": "aiohttp"}],
        indirect=True,
        ids=["loose", "strict", "aiohttp"],
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncHubClient) -> None:
        agent = await async_client.agents.create(
            headers={"name": "value"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            supported_languages=["string"],
            url="url",
        )
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncHubClient) -> None:
        agent = await async_client.agents.create(
            headers={"name": "value"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            supported_languages=["string"],
            url="url",
            description="description",
            stateful=True,
        )
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncHubClient) -> None:
        response = await async_client.agents.with_raw_response.create(
            headers={"name": "value"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            supported_languages=["string"],
            url="url",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncHubClient) -> None:
        async with async_client.agents.with_streaming_response.create(
            headers={"name": "value"},
            name="name",
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            supported_languages=["string"],
            url="url",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(Agent, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncHubClient) -> None:
        agent = await async_client.agents.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncHubClient) -> None:
        response = await async_client.agents.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncHubClient) -> None:
        async with async_client.agents.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(Agent, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `agent_id` but received ''",
        ):
            await async_client.agents.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncHubClient) -> None:
        agent = await async_client.agents.update(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncHubClient) -> None:
        agent = await async_client.agents.update(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            headers={"name": "value"},
            name="name",
            supported_languages=["string"],
            url="url",
        )
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncHubClient) -> None:
        response = await async_client.agents.with_raw_response.update(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncHubClient) -> None:
        async with async_client.agents.with_streaming_response.update(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(Agent, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `agent_id` but received ''",
        ):
            await async_client.agents.with_raw_response.update(
                agent_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncHubClient) -> None:
        agent = await async_client.agents.list()
        assert_matches_type(List[Agent], agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncHubClient) -> None:
        agent = await async_client.agents.list(
            project_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(List[Agent], agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHubClient) -> None:
        response = await async_client.agents.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(List[Agent], agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHubClient) -> None:
        async with async_client.agents.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(List[Agent], agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncHubClient) -> None:
        agent = await async_client.agents.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(type(None), agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncHubClient) -> None:
        response = await async_client.agents.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(type(None), agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncHubClient) -> None:
        async with async_client.agents.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(type(None), agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `agent_id` but received ''",
        ):
            await async_client.agents.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_bulk_delete(self, async_client: AsyncHubClient) -> None:
        agent = await async_client.agents.bulk_delete(
            agent_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(type(None), agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_bulk_delete(self, async_client: AsyncHubClient) -> None:
        response = await async_client.agents.with_raw_response.bulk_delete(
            agent_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(type(None), agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_bulk_delete(self, async_client: AsyncHubClient) -> None:
        async with async_client.agents.with_streaming_response.bulk_delete(
            agent_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(type(None), agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_generate_completion(self, async_client: AsyncHubClient) -> None:
        agent = await async_client.agents.generate_completion(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
        )
        assert_matches_type(AgentOutput, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_generate_completion(self, async_client: AsyncHubClient) -> None:
        response = await async_client.agents.with_raw_response.generate_completion(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(AgentOutput, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_generate_completion(self, async_client: AsyncHubClient) -> None:
        async with async_client.agents.with_streaming_response.generate_completion(
            agent_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(AgentOutput, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_generate_completion(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `agent_id` but received ''",
        ):
            await async_client.agents.with_raw_response.generate_completion(
                agent_id="",
                messages=[
                    {
                        "content": "content",
                        "role": "role",
                    }
                ],
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_test_connection(self, async_client: AsyncHubClient) -> None:
        agent = await async_client.agents.test_connection(
            url="url",
        )
        assert_matches_type(AgentOutput, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_test_connection_with_all_params(self, async_client: AsyncHubClient) -> None:
        agent = await async_client.agents.test_connection(
            url="url",
            headers={"foo": "string"},
        )
        assert_matches_type(AgentOutput, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_test_connection(self, async_client: AsyncHubClient) -> None:
        response = await async_client.agents.with_raw_response.test_connection(
            url="url",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(AgentOutput, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_test_connection(self, async_client: AsyncHubClient) -> None:
        async with async_client.agents.with_streaming_response.test_connection(
            url="url",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(AgentOutput, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_detect_statefulness(self, async_client: AsyncHubClient) -> None:
        result = await async_client.agents.detect_statefulness(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(AgentDetectStatefulness, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_detect_statefulness(self, async_client: AsyncHubClient) -> None:
        response = await async_client.agents.with_raw_response.detect_statefulness(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        result = await response.parse()
        assert_matches_type(AgentDetectStatefulness, result, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_detect_statefulness(self, async_client: AsyncHubClient) -> None:
        async with async_client.agents.with_streaming_response.detect_statefulness(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            result = await response.parse()
            assert_matches_type(AgentDetectStatefulness, result, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_detect_statefulness(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `agent_id` but received ''",
        ):
            await async_client.agents.with_raw_response.detect_statefulness(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_generate_description(self, async_client: AsyncHubClient) -> None:
        agent = await async_client.agents.generate_description(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(str, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_generate_description(self, async_client: AsyncHubClient) -> None:
        response = await async_client.agents.with_raw_response.generate_description(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(str, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_generate_description(self, async_client: AsyncHubClient) -> None:
        async with async_client.agents.with_streaming_response.generate_description(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(str, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_generate_description(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `agent_id` but received ''",
        ):
            await async_client.agents.with_raw_response.generate_description(
                "",
            )



from __future__ import annotations

from typing import List, Optional

import httpx

from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..types.task import (
    Task,
    TaskStatus,
    TaskPriority,
    TaskListParams,
    TaskCreateParams,
    TaskUpdateParams,
    TaskBulkDeleteParams,
)
from .._base_client import make_request_options
from ..types.common import APIResponse

__all__ = ["TasksResource", "AsyncTasksResource"]


class TasksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TasksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return TasksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TasksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return TasksResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        project_id: str,
        priority: Optional[TaskPriority] | Omit = omit,
        status: TaskStatus | Omit = omit,
        description: str,
        assignee_ids: SequenceNotStr[str] | Omit = omit,
        evaluation_result_id: Optional[str] | Omit = omit,
        dataset_test_case_id: Optional[str] | Omit = omit,
        probe_attempt_id: Optional[str] | Omit = omit,
        disable_test: bool | Omit = omit,
        hide_result: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Task:
        """
        Create a new task in the specified project.

        Parameters
        ----------
        project_id : str
            Project ID to create the task in.
        priority : Optional[TaskPriority] | Omit
            Priority of the task.
        status : TaskStatus | Omit
            Status of the task.
        description : str
            Description of the task to create.
        assignee_ids : SequenceNotStr[str] | Omit
            IDs of the users to assign the task to.
        evaluation_result_id : Optional[str] | Omit
            ID of the evaluation result to assign the task to.
        dataset_test_case_id : Optional[str] | Omit
            ID of the dataset test case to assign the task to.
        probe_attempt_id : Optional[str] | Omit
            ID of the probe attempt to assign the task to.
        disable_test : bool | Omit
            Whether to disable the test.
        hide_result : bool | Omit
            Whether to hide the result.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Task
            The newly created task.
        """
        response = self._post(
            "/v2/tasks",
            body=maybe_transform(
                {
                    "project_id": project_id,
                    "priority": priority,
                    "status": status,
                    "description": description,
                    "assignee_ids": assignee_ids,
                    "evaluation_result_id": evaluation_result_id,
                    "dataset_test_case_id": dataset_test_case_id,
                    "probe_attempt_id": probe_attempt_id,
                    "disable_test": disable_test,
                    "hide_result": hide_result,
                },
                TaskCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Task],
        )

        return self._unwrap(response)

    def retrieve(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Task:
        """
        Retrieve a task by its ID.

        Parameters
        ----------
        task_id : str
            ID of the task to retrieve.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Task
            The requested task.

        Raises
        ------
        ValueError
            If ``task_id`` is empty.
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        response = self._get(
            f"/v2/tasks/{task_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Task],
        )

        return self._unwrap(response)

    def update(
        self,
        task_id: str,
        *,
        status: Optional[TaskStatus] | Omit = omit,
        description: Optional[str] | Omit = omit,
        priority: Optional[TaskPriority] | Omit = omit,
        assignee_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        set_test_case_status: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Task:
        """
        Update an existing task's metadata and assignees.

        Parameters
        ----------
        task_id : str
            ID of the task to update.
        status : Optional[TaskStatus] | Omit
            Status of the task.
        description : Optional[str] | Omit
            Description of the task.
        priority : Optional[TaskPriority] | Omit
            Priority of the task.
        assignee_ids : Optional[SequenceNotStr[str]] | Omit
            IDs of the users to assign the task to.
        set_test_case_status : Optional[str] | Omit
            Status of the test case to set.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Task
            The updated task.

        Raises
        ------
        ValueError
            If ``task_id`` is empty.
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        response = self._patch(
            f"/v2/tasks/{task_id}",
            body=maybe_transform(
                {
                    "status": status,
                    "description": description,
                    "priority": priority,
                    "assignee_ids": assignee_ids,
                    "set_test_case_status": set_test_case_status,
                },
                TaskUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Task],
        )

        return self._unwrap(response)

    def list(
        self,
        *,
        project_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[Task]:
        """
        List all tasks for a project, ordered by creation date descending.

        Parameters
        ----------
        project_id : Optional[str] | Omit
            Project ID to list tasks for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[Task]
            A list of tasks for the specified project.
        """
        response = self._get(
            "/v2/tasks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "project_id": project_id,
                    },
                    TaskListParams,
                ),
            ),
            cast_to=APIResponse[List[Task]],
        )

        return self._unwrap(response)

    def delete(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a task by its ID.

        Parameters
        ----------
        task_id : str
            ID of the task to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``task_id`` is empty.
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        response = self._delete(
            f"/v2/tasks/{task_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def bulk_delete(
        self,
        *,
        task_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete multiple tasks at once.

        Parameters
        ----------
        task_ids : SequenceNotStr[str]
            IDs of the tasks to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None
        """
        response = self._delete(
            "/v2/tasks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"task_ids": task_ids}, TaskBulkDeleteParams),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)


class AsyncTasksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTasksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTasksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTasksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AsyncTasksResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        project_id: str,
        priority: Optional[TaskPriority] | Omit = omit,
        status: TaskStatus | Omit = omit,
        description: str,
        assignee_ids: SequenceNotStr[str] | Omit = omit,
        evaluation_result_id: Optional[str] | Omit = omit,
        dataset_test_case_id: Optional[str] | Omit = omit,
        probe_attempt_id: Optional[str] | Omit = omit,
        disable_test: bool | Omit = omit,
        hide_result: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Task:
        """
        Create a new task in the specified project.

        Parameters
        ----------
        project_id : str
            Project ID to create the task in.
        priority : Optional[TaskPriority] | Omit
            Priority of the task.
        status : TaskStatus | Omit
            Status of the task.
        description : str
            Description of the task to create.
        assignee_ids : SequenceNotStr[str] | Omit
            IDs of the users to assign the task to.
        evaluation_result_id : Optional[str] | Omit
            ID of the evaluation result to assign the task to.
        dataset_test_case_id : Optional[str] | Omit
            ID of the dataset test case to assign the task to.
        probe_attempt_id : Optional[str] | Omit
            ID of the probe attempt to assign the task to.
        disable_test : bool | Omit
            Whether to disable the test.
        hide_result : bool | Omit
            Whether to hide the result.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Task
            The newly created task.
        """
        response = await self._post(
            "/v2/tasks",
            body=await async_maybe_transform(
                {
                    "project_id": project_id,
                    "priority": priority,
                    "status": status,
                    "description": description,
                    "assignee_ids": assignee_ids,
                    "evaluation_result_id": evaluation_result_id,
                    "dataset_test_case_id": dataset_test_case_id,
                    "probe_attempt_id": probe_attempt_id,
                    "disable_test": disable_test,
                    "hide_result": hide_result,
                },
                TaskCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Task],
        )

        return self._unwrap(response)

    async def retrieve(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Task:
        """
        Retrieve a task by its ID.

        Parameters
        ----------
        task_id : str
            ID of the task to retrieve.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Task
            The requested task.

        Raises
        ------
        ValueError
            If ``task_id`` is empty.
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        response = await self._get(
            f"/v2/tasks/{task_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Task],
        )

        return self._unwrap(response)

    async def update(
        self,
        task_id: str,
        *,
        status: Optional[TaskStatus] | Omit = omit,
        description: Optional[str] | Omit = omit,
        priority: Optional[TaskPriority] | Omit = omit,
        assignee_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        set_test_case_status: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Task:
        """
        Update an existing task's metadata and assignees.

        Parameters
        ----------
        task_id : str
            ID of the task to update.
        status : Optional[TaskStatus] | Omit
            Status of the task.
        description : Optional[str] | Omit
            Description of the task.
        priority : Optional[TaskPriority] | Omit
            Priority of the task.
        assignee_ids : Optional[SequenceNotStr[str]] | Omit
            IDs of the users to assign the task to.
        set_test_case_status : Optional[str] | Omit
            Status of the test case to set.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        Task
            The updated task.

        Raises
        ------
        ValueError
            If ``task_id`` is empty.
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        response = await self._patch(
            f"/v2/tasks/{task_id}",
            body=await async_maybe_transform(
                {
                    "status": status,
                    "description": description,
                    "priority": priority,
                    "assignee_ids": assignee_ids,
                    "set_test_case_status": set_test_case_status,
                },
                TaskUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Task],
        )

        return self._unwrap(response)

    async def list(
        self,
        *,
        project_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[Task]:
        """
        List all tasks for a project, ordered by creation date descending.

        Parameters
        ----------
        project_id : Optional[str] | Omit
            Project ID to list tasks for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[Task]
            A list of tasks for the specified project.
        """
        response = await self._get(
            "/v2/tasks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "project_id": project_id,
                    },
                    TaskListParams,
                ),
            ),
            cast_to=APIResponse[List[Task]],
        )

        return self._unwrap(response)

    async def delete(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a task by its ID.

        Parameters
        ----------
        task_id : str
            ID of the task to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``task_id`` is empty.
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        response = await self._delete(
            f"/v2/tasks/{task_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def bulk_delete(
        self,
        *,
        task_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete multiple tasks at once.

        Parameters
        ----------
        task_ids : SequenceNotStr[str]
            IDs of the tasks to delete.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        None
        """
        response = await self._delete(
            "/v2/tasks",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"task_ids": task_ids}, TaskBulkDeleteParams),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)


class TasksResourceWithRawResponse:
    def __init__(self, tasks: TasksResource) -> None:
        self._tasks = tasks

        self.create = to_raw_response_wrapper(
            tasks.create,
        )
        self.retrieve = to_raw_response_wrapper(
            tasks.retrieve,
        )
        self.update = to_raw_response_wrapper(
            tasks.update,
        )
        self.list = to_raw_response_wrapper(
            tasks.list,
        )
        self.delete = to_raw_response_wrapper(
            tasks.delete,
        )
        self.bulk_delete = to_raw_response_wrapper(
            tasks.bulk_delete,
        )


class AsyncTasksResourceWithRawResponse:
    def __init__(self, tasks: AsyncTasksResource) -> None:
        self._tasks = tasks

        self.create = async_to_raw_response_wrapper(
            tasks.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            tasks.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            tasks.update,
        )
        self.list = async_to_raw_response_wrapper(
            tasks.list,
        )
        self.delete = async_to_raw_response_wrapper(
            tasks.delete,
        )
        self.bulk_delete = async_to_raw_response_wrapper(
            tasks.bulk_delete,
        )


class TasksResourceWithStreamingResponse:
    def __init__(self, tasks: TasksResource) -> None:
        self._tasks = tasks

        self.create = to_streamed_response_wrapper(
            tasks.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            tasks.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            tasks.update,
        )
        self.list = to_streamed_response_wrapper(
            tasks.list,
        )
        self.delete = to_streamed_response_wrapper(
            tasks.delete,
        )
        self.bulk_delete = to_streamed_response_wrapper(
            tasks.bulk_delete,
        )


class AsyncTasksResourceWithStreamingResponse:
    def __init__(self, tasks: AsyncTasksResource) -> None:
        self._tasks = tasks

        self.create = async_to_streamed_response_wrapper(
            tasks.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            tasks.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            tasks.update,
        )
        self.list = async_to_streamed_response_wrapper(
            tasks.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            tasks.delete,
        )
        self.bulk_delete = async_to_streamed_response_wrapper(
            tasks.bulk_delete,
        )



"""Agent domain types."""

from typing import Any, Dict, Iterable, Optional, TypedDict, cast
from datetime import datetime
from typing_extensions import Required

from pydantic import Field, AliasChoices, model_validator

from .chat import Header, ChatMessage, HeaderParam, ChatMessageParam
from .._types import SequenceNotStr
from .._models import BaseModel
from .execution import ExecutionError, ExecutionErrorParam

__all__ = [
    "Agent",
    "AgentReference",
    "AgentOutput",
    "AgentOutputParam",
    "MinimalAgent",
    "MinimalAgentParam",
    "AgentDetectStatefulness",
    "AgentListParams",
    "AgentCreateParams",
    "AgentUpdateParams",
    "AgentBulkDeleteParams",
    "AgentTestConnectionParams",
    "AgentGenerateCompletionParams",
    "AgentAutofillDescriptionParams",
    "AgentDetectStatefulnessParams",
]


# ---------------------------------------------------------------------------
# Models
# ---------------------------------------------------------------------------


class Agent(BaseModel):
    id: str
    created_at: datetime
    description: Optional[str] = None
    headers: Dict[str, str] = Field(default_factory=dict)
    name: str
    project_id: str
    stateful: bool
    supported_languages: list[str]
    updated_at: datetime
    url: str

    @model_validator(mode="before")
    @classmethod
    def _normalize_headers(cls, data: Any) -> Any:  # noqa: ANN401
        if not isinstance(data, dict):
            return data

        d = cast(Dict[str, Any], data)
        raw_headers = d.get("headers")

        # Already a mapping or missing: nothing to do
        if raw_headers is None or isinstance(raw_headers, dict):
            return d

        # Convert list of Header models or dicts into a dict[str, str]
        if isinstance(raw_headers, list):
            headers_dict: Dict[str, str] = {}
            for item in cast(list[Any], raw_headers):
                if isinstance(item, Header):
                    headers_dict[item.name] = item.value
                elif isinstance(item, dict):
                    item_typed = cast(Dict[str, Any], item)
                    name = item_typed.get("name")
                    value = item_typed.get("value")
                    if isinstance(name, str) and isinstance(value, str):
                        headers_dict[name] = value

            return {**d, "headers": headers_dict}

        return d


class AgentReference(BaseModel):
    id: str
    name: str


class AgentOutput(BaseModel):
    response: Optional[ChatMessage] = Field(
        default=None,
        validation_alias=AliasChoices("response", "message"),
    )
    error: Optional[ExecutionError] = None
    metadata: Optional[Dict[str, object]] = None


class AgentOutputParam(TypedDict, total=False):
    response: Required[Optional[ChatMessageParam]]
    error: Optional[ExecutionErrorParam]
    metadata: Dict[str, object]


class MinimalAgent(BaseModel):
    name: str
    description: Optional[str] = None


class MinimalAgentParam(TypedDict, total=False):
    name: Required[str]
    description: Optional[str]


class AgentDetectStatefulness(BaseModel):
    stateful: bool


# ---------------------------------------------------------------------------
# Params
# ---------------------------------------------------------------------------


class AgentListParams(TypedDict, total=False):
    project_id: Optional[str]


class AgentCreateParams(TypedDict, total=False):
    headers: Required[Iterable[HeaderParam]]
    name: Required[str]
    project_id: Required[str]
    supported_languages: Required[SequenceNotStr[str]]
    url: Required[str]
    description: Optional[str]
    stateful: Optional[bool]


class AgentUpdateParams(TypedDict, total=False):
    description: Optional[str]
    headers: Optional[Iterable[HeaderParam]]
    name: Optional[str]
    supported_languages: Optional[SequenceNotStr[str]]
    url: Optional[str]


class AgentBulkDeleteParams(TypedDict, total=False):
    agent_ids: Required[SequenceNotStr[str]]


class AgentTestConnectionParams(TypedDict, total=False):
    url: Required[str]
    headers: Dict[str, str]


class AgentGenerateCompletionParams(TypedDict, total=False):
    messages: Required[Iterable[ChatMessageParam]]


class AgentAutofillDescriptionParams(TypedDict, total=False):
    pass


class AgentDetectStatefulnessParams(TypedDict, total=False):
    pass



"""Scenario domain types."""

from typing import Any, Dict, List, Optional, TypedDict
from datetime import datetime
from typing_extensions import Required

from .._types import SequenceNotStr
from .._models import BaseModel

__all__ = [
    "Scenario",
    "ScenarioPreview",
    "ScenarioCreateParams",
    "ScenarioUpdateParams",
    "ScenarioPreviewParams",
]


# ---------------------------------------------------------------------------
# Models
# ---------------------------------------------------------------------------


class Scenario(BaseModel):
    id: str
    created_at: datetime
    updated_at: datetime
    name: str
    description: Optional[str] = None
    rules: SequenceNotStr[str]


class ScenarioPreview(BaseModel):
    conversation: List[Dict[str, Any]]
    generated_rules: Optional[List[str]]


# ---------------------------------------------------------------------------
# Params
# ---------------------------------------------------------------------------


class ScenarioCreateParams(TypedDict, total=False):
    name: Required[str]
    description: Required[str]
    rules: SequenceNotStr[str]


class ScenarioUpdateParams(TypedDict, total=False):
    name: Optional[str]
    description: Optional[str]
    rules: Optional[SequenceNotStr[str]]


class ScenarioPreviewParams(TypedDict, total=False):
    description: Required[str]
    rules: SequenceNotStr[str]



"""Type definitions for the Giskard Hub API.

Types are organized by domain into consolidated modules:

- common: Generic response wrappers, filter/order helpers, TaskState, TaskProgress
- chat: ChatMessage, Header and their param variants
- user: User, UserReference
- execution: ExecutionError, execution status types
- agent: Agent, AgentOutput, MinimalAgent and params
- check: Check, assertions, annotations and params
- audit: Audit, AuditDisplay and params
- task: Task, TaskStatus, TaskPriority, references and params
- dataset: Dataset, DatasetSubset and params
- knowledge_base: KnowledgeBase, document types and params
- evaluation: Evaluation, Metric, TestCaseEvaluation, result params
- scan: Scan, ScanCategory, ScanProbe types and params
- scenario: Scenario, ScenarioPreview and params
- scheduled_evaluation: ScheduledEvaluation, FrequencyOption and params
- test_case: TestCase, TestCaseComment, comment params
- playground_chat: PlaygroundChat and params
- project: Project and params
"""

# -- common ----------------------------------------------------------------
from .common import (  # noqa: I001
    APIResponse as APIResponse,
    APIPaginatedMetadata as APIPaginatedMetadata,
    APIPaginatedResponse as APIPaginatedResponse,
    APIResponseWithIncluded as APIResponseWithIncluded,
    DateRangeFilterValueParam as DateRangeFilterValueParam,
    FilterValueParam as FilterValueParam,
    ListFilterValueParam as ListFilterValueParam,
    OrderByParam as OrderByParam,
    PaginatedMetadata as PaginatedMetadata,
    TaskProgress as TaskProgress,
    TaskProgressParam as TaskProgressParam,
    TaskState as TaskState,
)

# -- chat ------------------------------------------------------------------
from .chat import (
    ChatMessage as ChatMessage,
    ChatMessageParam as ChatMessageParam,
    ChatMessageWithMetadata as ChatMessageWithMetadata,
    ChatMessageWithMetadataParam as ChatMessageWithMetadataParam,
    Header as Header,
    HeaderParam as HeaderParam,
)

# -- user ------------------------------------------------------------------
from .user import (
    User as User,
    UserReference as UserReference,
)

# -- execution -------------------------------------------------------------
from .execution import (
    ErrorExecutionStatus as ErrorExecutionStatus,
    ErrorExecutionStatusParam as ErrorExecutionStatusParam,
    ExecutionError as ExecutionError,
    ExecutionErrorParam as ExecutionErrorParam,
    SuccessExecutionStatus as SuccessExecutionStatus,
    SuccessExecutionStatusParam as SuccessExecutionStatusParam,
)

# -- agent -----------------------------------------------------------------
from .agent import (
    Agent as Agent,
    AgentAutofillDescriptionParams as AgentAutofillDescriptionParams,
    AgentBulkDeleteParams as AgentBulkDeleteParams,
    AgentCreateParams as AgentCreateParams,
    AgentDetectStatefulnessParams as AgentDetectStatefulnessParams,
    AgentGenerateCompletionParams as AgentGenerateCompletionParams,
    AgentListParams as AgentListParams,
    AgentOutput as AgentOutput,
    AgentOutputParam as AgentOutputParam,
    AgentReference as AgentReference,
    AgentTestConnectionParams as AgentTestConnectionParams,
    AgentUpdateParams as AgentUpdateParams,
    AgentDetectStatefulness as AgentDetectStatefulness,
    MinimalAgent as MinimalAgent,
    MinimalAgentParam as MinimalAgentParam,
)

# -- check -----------------------------------------------------------------
from .check import (
    Check as Check,
    CheckBulkDeleteParams as CheckBulkDeleteParams,
    CheckConfig as CheckConfig,
    CheckConfigParam as CheckConfigParam,
    CheckCreateParams as CheckCreateParams,
    CheckListParams as CheckListParams,
    CheckResult as CheckResult,
    CheckType as CheckType,
    CheckTypeParam as CheckTypeParam,
    CheckUpdateParams as CheckUpdateParams,
    ConformityParams as ConformityParams,
    ConformityParamsParam as ConformityParamsParam,
    CorrectnessParams as CorrectnessParams,
    CorrectnessParamsParam as CorrectnessParamsParam,
    GroundednessParams as GroundednessParams,
    GroundednessParamsParam as GroundednessParamsParam,
    MetadataParams as MetadataParams,
    MetadataParamsParam as MetadataParamsParam,
    OutputAnnotation as OutputAnnotation,
    SemanticSimilarityParams as SemanticSimilarityParams,
    SemanticSimilarityParamsParam as SemanticSimilarityParamsParam,
    StringMatchParams as StringMatchParams,
    StringMatchParamsParam as StringMatchParamsParam,
    TestCaseCheckConfig as TestCaseCheckConfig,
    TestCaseCheckConfigParam as TestCaseCheckConfigParam,
)

# -- audit -----------------------------------------------------------------
from .audit import (
    ActionType as ActionType,
    Audit as Audit,
    AuditDiffItem as AuditDiffItem,
    AuditDiffKind as AuditDiffKind,
    AuditDisplay as AuditDisplay,
    AuditFiltersParam as AuditFiltersParam,
    AuditListEntityParams as AuditListEntityParams,
    AuditOrderByParam as AuditOrderByParam,
    AuditSearchParams as AuditSearchParams,
)

# -- task ------------------------------------------------------------------
from .task import (
    Task as Task,
    TaskBulkDeleteParams as TaskBulkDeleteParams,
    TaskCreateParams as TaskCreateParams,
    TaskListParams as TaskListParams,
    TaskPriority as TaskPriority,
    TaskStatus as TaskStatus,
    TaskUpdateParams as TaskUpdateParams,
)

# -- dataset ---------------------------------------------------------------
from .dataset import (
    Dataset as Dataset,
    DatasetBulkDeleteParams as DatasetBulkDeleteParams,
    DatasetCreateParams as DatasetCreateParams,
    DatasetGenerateAdversarialParams as DatasetGenerateAdversarialParams,
    DatasetGenerateDocumentBasedParams as DatasetGenerateDocumentBasedParams,
    DatasetGenerateScenarioBasedParams as DatasetGenerateScenarioBasedParams,
    DatasetImportParams as DatasetImportParams,
    DatasetListParams as DatasetListParams,
    DatasetReference as DatasetReference,
    DatasetSearchTestCasesParams as DatasetSearchTestCasesParams,
    DatasetSubset as DatasetSubset,
    DatasetSubsetParam as DatasetSubsetParam,
    DatasetUpdateParams as DatasetUpdateParams,
    TestCaseFiltersParam as TestCaseFiltersParam,
    TestCaseOrderByParam as TestCaseOrderByParam,
)

# -- knowledge_base --------------------------------------------------------
from .knowledge_base import (
    KnowledgeBase as KnowledgeBase,
    KnowledgeBaseReference as KnowledgeBaseReference,
    KnowledgeBaseBulkDeleteParams as KnowledgeBaseBulkDeleteParams,
    KnowledgeBaseCreateParams as KnowledgeBaseCreateParams,
    KnowledgeBaseDocumentDetail as KnowledgeBaseDocumentDetail,
    KnowledgeBaseDocumentFiltersParam as KnowledgeBaseDocumentFiltersParam,
    KnowledgeBaseDocumentOrderByParam as KnowledgeBaseDocumentOrderByParam,
    KnowledgeBaseDocumentRow as KnowledgeBaseDocumentRow,
    KnowledgeBaseListParams as KnowledgeBaseListParams,
    KnowledgeBaseSearchDocumentsParams as KnowledgeBaseSearchDocumentsParams,
    KnowledgeBaseUpdateParams as KnowledgeBaseUpdateParams,
    Topic as Topic,
)

# -- evaluation ------------------------------------------------------------
from .evaluation import (
    DivergenceWarning as DivergenceWarning,
    Evaluation as Evaluation,
    EvaluationReference as EvaluationReference,
    EvaluationBulkDeleteParams as EvaluationBulkDeleteParams,
    EvaluationCreateLocalParams as EvaluationCreateLocalParams,
    EvaluationCreateParams as EvaluationCreateParams,
    EvaluationListParams as EvaluationListParams,
    EvaluationRetrieveParams as EvaluationRetrieveParams,
    EvaluationRunSingleParams as EvaluationRunSingleParams,
    EvaluationUpdateParams as EvaluationUpdateParams,
    FailureCategory as FailureCategory,
    FailureCategoryParam as FailureCategoryParam,
    Metric as Metric,
    ResultFiltersParam as ResultFiltersParam,
    ResultListParams as ResultListParams,
    ResultOrderByParam as ResultOrderByParam,
    ResultRetrieveParams as ResultRetrieveParams,
    ResultSearchParams as ResultSearchParams,
    ResultSubmitLocalOutputParams as ResultSubmitLocalOutputParams,
    ResultUpdateParams as ResultUpdateParams,
    ResultUpdateVisibilityParams as ResultUpdateVisibilityParams,
    TestCaseEvaluation as TestCaseEvaluation,
    TestCaseEvaluationReference as TestCaseEvaluationReference,
)

# -- scan ------------------------------------------------------------------
from .scan import (
    ScanProbeAttemptUpdateParams as ScanProbeAttemptUpdateParams,
    ReviewStatus as ReviewStatus,
    ScanBulkDeleteParams as ScanBulkDeleteParams,
    ScanCategory as ScanCategory,
    ScanCreateParams as ScanCreateParams,
    ScanListParams as ScanListParams,
    ScanProbeAttempt as ScanProbeAttempt,
    ScanProbeAttemptReference as ScanProbeAttemptReference,
    ScanProbe as ScanProbe,
    Scan as Scan,
    ScanRetrieveParams as ScanRetrieveParams,
    Severity as Severity,
)

# -- scenario --------------------------------------------------------------
from .scenario import (
    Scenario as Scenario,
    ScenarioCreateParams as ScenarioCreateParams,
    ScenarioPreview as ScenarioPreview,
    ScenarioPreviewParams as ScenarioPreviewParams,
    ScenarioUpdateParams as ScenarioUpdateParams,
)

# -- scheduled_evaluation --------------------------------------------------
from .scheduled_evaluation import (
    FrequencyOption as FrequencyOption,
    ScheduledEvaluation as ScheduledEvaluation,
    ScheduledEvaluationBulkDeleteParams as ScheduledEvaluationBulkDeleteParams,
    ScheduledEvaluationCreateParams as ScheduledEvaluationCreateParams,
    ScheduledEvaluationListEvaluationsParams as ScheduledEvaluationListEvaluationsParams,
    ScheduledEvaluationListParams as ScheduledEvaluationListParams,
    ScheduledEvaluationRetrieveParams as ScheduledEvaluationRetrieveParams,
    ScheduledEvaluationUpdateParams as ScheduledEvaluationUpdateParams,
)

# -- test_case -------------------------------------------------------------
from .test_case import (
    BulkMoveTestCasesParams as BulkMoveTestCasesParams,
    CommentAddParams as CommentAddParams,
    CommentEditParams as CommentEditParams,
    TestCase as TestCase,
    TestCaseReference as TestCaseReference,
    TestCaseBulkDeleteParams as TestCaseBulkDeleteParams,
    TestCaseBulkUpdateParams as TestCaseBulkUpdateParams,
    TestCaseComment as TestCaseComment,
    TestCaseCreateParams as TestCaseCreateParams,
    TestCaseUpdateParams as TestCaseUpdateParams,
)

# -- playground_chat -------------------------------------------------------
from .playground_chat import (
    PlaygroundChat as PlaygroundChat,
    PlaygroundChatBulkDeleteParams as PlaygroundChatBulkDeleteParams,
    PlaygroundChatListParams as PlaygroundChatListParams,
    PlaygroundChatRetrieveParams as PlaygroundChatRetrieveParams,
)

# -- project ---------------------------------------------------------------
from .project import (
    Project as Project,
    ProjectBulkDeleteParams as ProjectBulkDeleteParams,
    ProjectCreateParams as ProjectCreateParams,
    ProjectUpdateParams as ProjectUpdateParams,
)


__all__ = [
    # common
    "APIResponse",
    "APIPaginatedMetadata",
    "APIPaginatedResponse",
    "APIResponseWithIncluded",
    "PaginatedMetadata",
    "TaskState",
    "TaskProgress",
    "TaskProgressParam",
    "FilterValueParam",
    "ListFilterValueParam",
    "DateRangeFilterValueParam",
    "OrderByParam",
    # chat
    "ChatMessage",
    "ChatMessageParam",
    "ChatMessageWithMetadata",
    "ChatMessageWithMetadataParam",
    "Header",
    "HeaderParam",
    # user
    "User",
    "UserReference",
    # execution
    "ExecutionError",
    "ExecutionErrorParam",
    "ErrorExecutionStatus",
    "ErrorExecutionStatusParam",
    "SuccessExecutionStatus",
    "SuccessExecutionStatusParam",
    # agent
    "Agent",
    "AgentReference",
    "AgentOutput",
    "AgentOutputParam",
    "MinimalAgent",
    "MinimalAgentParam",
    "AgentDetectStatefulness",
    "AgentListParams",
    "AgentCreateParams",
    "AgentUpdateParams",
    "AgentBulkDeleteParams",
    "AgentTestConnectionParams",
    "AgentGenerateCompletionParams",
    "AgentAutofillDescriptionParams",
    "AgentDetectStatefulnessParams",
    # check
    "Check",
    "CheckResult",
    "CheckType",
    "CheckTypeParam",
    "ConformityParams",
    "ConformityParamsParam",
    "CorrectnessParams",
    "CorrectnessParamsParam",
    "GroundednessParams",
    "GroundednessParamsParam",
    "StringMatchParams",
    "StringMatchParamsParam",
    "SemanticSimilarityParams",
    "SemanticSimilarityParamsParam",
    "MetadataParams",
    "MetadataParamsParam",
    "OutputAnnotation",
    "CheckConfig",
    "CheckConfigParam",
    "TestCaseCheckConfig",
    "TestCaseCheckConfigParam",
    "CheckListParams",
    "CheckCreateParams",
    "CheckUpdateParams",
    "CheckBulkDeleteParams",
    # audit
    "ActionType",
    "AuditDiffKind",
    "AuditDiffItem",
    "Audit",
    "AuditDisplay",
    "AuditListEntityParams",
    "AuditSearchParams",
    "AuditOrderByParam",
    "AuditFiltersParam",
    # task
    "Task",
    "TaskStatus",
    "TaskPriority",
    "TaskListParams",
    "TaskCreateParams",
    "TaskUpdateParams",
    "TaskBulkDeleteParams",
    # dataset
    "Dataset",
    "DatasetReference",
    "DatasetSubset",
    "DatasetSubsetParam",
    "DatasetListParams",
    "DatasetCreateParams",
    "DatasetImportParams",
    "DatasetUpdateParams",
    "DatasetBulkDeleteParams",
    "DatasetSearchTestCasesParams",
    "TestCaseOrderByParam",
    "TestCaseFiltersParam",
    "DatasetGenerateAdversarialParams",
    "DatasetGenerateDocumentBasedParams",
    "DatasetGenerateScenarioBasedParams",
    # knowledge base
    "KnowledgeBase",
    "KnowledgeBaseReference",
    "Topic",
    "KnowledgeBaseDocumentRow",
    "KnowledgeBaseDocumentDetail",
    "KnowledgeBaseListParams",
    "KnowledgeBaseCreateParams",
    "KnowledgeBaseUpdateParams",
    "KnowledgeBaseBulkDeleteParams",
    "KnowledgeBaseSearchDocumentsParams",
    "KnowledgeBaseDocumentOrderByParam",
    "KnowledgeBaseDocumentFiltersParam",
    # evaluation
    "Metric",
    "DivergenceWarning",
    "Evaluation",
    "EvaluationReference",
    "EvaluationListParams",
    "EvaluationCreateParams",
    "EvaluationUpdateParams",
    "EvaluationRetrieveParams",
    "EvaluationRunSingleParams",
    "EvaluationCreateLocalParams",
    "EvaluationBulkDeleteParams",
    "FailureCategory",
    "FailureCategoryParam",
    "TestCaseEvaluation",
    "TestCaseEvaluationReference",
    "ResultListParams",
    "ResultSearchParams",
    "ResultUpdateParams",
    "ResultRetrieveParams",
    "ResultUpdateVisibilityParams",
    "ResultSubmitLocalOutputParams",
    "ResultFiltersParam",
    "ResultOrderByParam",
    # scan
    "Scan",
    "ScanCategory",
    "ScanListParams",
    "ScanCreateParams",
    "ScanRetrieveParams",
    "ScanBulkDeleteParams",
    "Severity",
    "ReviewStatus",
    "ScanProbe",
    "ScanProbeAttempt",
    "ScanProbeAttemptReference",
    "ScanProbeAttemptUpdateParams",
    # scenario
    "Scenario",
    "ScenarioPreview",
    "ScenarioCreateParams",
    "ScenarioUpdateParams",
    "ScenarioPreviewParams",
    # scheduled evaluation
    "FrequencyOption",
    "ScheduledEvaluation",
    "ScheduledEvaluationListParams",
    "ScheduledEvaluationCreateParams",
    "ScheduledEvaluationUpdateParams",
    "ScheduledEvaluationRetrieveParams",
    "ScheduledEvaluationBulkDeleteParams",
    "ScheduledEvaluationListEvaluationsParams",
    # test case
    "TestCase",
    "TestCaseReference",
    "TestCaseComment",
    "BulkMoveTestCasesParams",
    "TestCaseCreateParams",
    "TestCaseUpdateParams",
    "TestCaseBulkDeleteParams",
    "TestCaseBulkUpdateParams",
    "CommentAddParams",
    "CommentEditParams",
    # playground chat
    "PlaygroundChat",
    "PlaygroundChatListParams",
    "PlaygroundChatRetrieveParams",
    "PlaygroundChatBulkDeleteParams",
    # project
    "Project",
    "ProjectCreateParams",
    "ProjectUpdateParams",
    "ProjectBulkDeleteParams",
]
